@@ -111,6 +111,7 @@ int wsmc_transport_init(WsManClient *cl, void *arg)
 	while (InterlockedExchange(&cl->lock_session_handle, 1L));
 	if (cl->session_handle != NULL) {
 		cl->lock_session_handle = 0L;
+		u_free(agent);
 		return 0;
 	}
 	if(!cl->proxy_data.proxy){
@@ -168,6 +169,7 @@ static void *init_win_transport(WsManClient * cl)
 	}
 	if (cl->session_handle == NULL) {
 		error("could not initialize session");
+		u_free(host);
 		return NULL;
 	}
 
@@ -301,7 +303,7 @@ wsmc_handler(WsManClient * cl, WsXmlDocH rqstDoc, void *user_data)
 	char pszAnsi[128];
 	wchar_t *pwsz;
 
-	PCCERT_CONTEXT certificate;
+	PCCERT_CONTEXT certificate = NULL;
 	wchar_t *proxy_username;
 	wchar_t *proxy_password;
 	if (cl->session_handle == NULL && wsmc_transport_init(cl, NULL)) {
@@ -373,6 +375,12 @@ wsmc_handler(WsManClient * cl, WsXmlDocH rqstDoc, void *user_data)
 	if(cl->proxy_data.proxy_username)
 	{
 		proxy_username = convert_to_unicode(cl->proxy_data.proxy_username);
+		if (proxy_username == NULL) {
+			lastErr = GetLastError();
+			dwStatusCode = 400;
+			error("could not convert proxy_username to unicode");
+			goto DONE;
+		}
 		bResults = WinHttpSetOption(request, WINHTTP_OPTION_PROXY_USERNAME,
 				proxy_username, wcslen(proxy_username));
 		u_free(proxy_username);
@@ -387,9 +395,15 @@ wsmc_handler(WsManClient * cl, WsXmlDocH rqstDoc, void *user_data)
 	if(cl->proxy_data.proxy_password)
 	{
 		proxy_password = convert_to_unicode(cl->proxy_data.proxy_password);
+		if (proxy_password == NULL) {
+			lastErr = GetLastError();
+			dwStatusCode = 400;
+			error("could not convert proxy_password to unicode");
+			goto DONE;
+		}
 		bResults = WinHttpSetOption(request, WINHTTP_OPTION_PROXY_PASSWORD,
 				proxy_password, wcslen(proxy_password));
-		u_free(proxy_password);
+		u_cleanfreew(proxy_password);
 		if (!bResults)
 		{
 			lastErr = GetLastError();
@@ -499,6 +513,12 @@ wsmc_handler(WsManClient * cl, WsXmlDocH rqstDoc, void *user_data)
 					pwd = convert_to_unicode(cl->data.pwd);
 					usr = convert_to_unicode(cl->data.user);
 					if ((pwd == NULL) || (usr == NULL)) {
+						if (pwd != NULL) {
+							u_cleanfreew(pwd);
+						}
+						if (usr != NULL) {
+							u_free(usr);
+						}
 						bDone = TRUE;
 						bResults = 0;
 						break;
@@ -508,7 +528,7 @@ wsmc_handler(WsManClient * cl, WsXmlDocH rqstDoc, void *user_data)
 							dwSelectedScheme,
 							usr, pwd,
 							NULL);
-					u_free(pwd);
+					u_cleanfreew(pwd);
 					u_free(usr);
 				}
 				if (cleanup_request_data(request)) {
@@ -598,6 +618,12 @@ wsmc_handler(WsManClient * cl, WsXmlDocH rqstDoc, void *user_data)
 				pwd = convert_to_unicode(cl->data.pwd);
 				usr = convert_to_unicode(cl->data.user);
 				if ((pwd == NULL) || (usr == NULL)) {
+					if (pwd != NULL) {
+						u_cleanfreew(pwd);
+					}
+					if (usr != NULL) {
+						u_free(usr);
+					}
 					bDone = TRUE;
 					bResults = 0;
 					break;
@@ -607,7 +633,7 @@ wsmc_handler(WsManClient * cl, WsXmlDocH rqstDoc, void *user_data)
 						dwSelectedScheme,
 						usr, pwd,
 						NULL);
-				u_free(pwd);
+				u_cleanfreew(pwd);
 				u_free(usr);
 			}
 			if (cleanup_request_data(request)) {
@@ -727,7 +753,7 @@ BOOL find_cert(const _TCHAR * oid,
 	if ( !(hStoreHandle = CertOpenStore(
 					CERT_STORE_PROV_SYSTEM,          // The store provider type
 					0,                               // The encoding type is not needed
-					NULL,                            // Use the default HCRYPTPROV
+					(HCRYPTPROV_LEGACY)NULL,         // Use the default HCRYPTPROV
 					flags,  // Set the store location in a registry location
 					L"MY"                            // The store name as a Unicode string
 					)))
